<?php

/**
 * @file
 * Content creation / editing handler
 */

function drupad_content_forms() {
  switch (arg(2)) {
    // ?q=drupad/content-forms/node/[nid]
    case 'node':
      module_load_include('inc', 'drupad', 'drupad.forms');
      $data = drupad_content_get_form_for_node(arg(3));
      break;
    // ?q=drupad/content-forms/node-add/[content_type]
    case 'node-add':
      module_load_include('inc', 'drupad', 'drupad.forms');
      $data = drupad_content_get_form_for_node_type(arg(3));
      break;
    // ?q=drupad/content-forms
    default:
      $data = drupad_content_get_content_types();
      break;
  }

  drupad_serve_response($data);
}

/**
 * Get all content types the current user can create
 *
 * @return structured response array
 */
function drupad_content_get_content_types() {
  $types = node_get_types('types', NULL, FALSE);

  $content_types = array();
  $content_types_to_exclude = variable_get('drupad_content_types_excluded_node_add', array());

  foreach ($types as $key => $type) {
    if (node_access('create', $type->type) && !in_array($type->type, $content_types_to_exclude)) {
      $content_types[] = $type;
    }
  }

  $data['meta'] = array();
  $data['content'] = $content_types;

  return $data;
}

/**
 * We need to overwrite node_form_submit() to redirect node form
 * submissions to our own node view callback once submission succeed
 * @see drupad_node_form_submit()
 */
function drupad_form_alter(&$form, $form_state, $form_id) {
  // We only alter forms when the iOS app is requesting a node form
  if ($form['#id'] == 'node-form' && arg(0) == 'drupad' && arg(1) == 'content-forms') {
    $form['buttons']['submit']['#submit'][] = 'drupad_node_form_submit';
    /////////////////////////////////////////////////////
    // We add labels to upload module form components
    // since they are presented in a table when in HTML format
    if (module_exists('upload') && isset($form['attachments']['wrapper']['files'])) {
      // Upload module creates a draggable table. Labels are placed in that table
      // thus we need to re-set them to obtain something understandable on the app.
      foreach ($form['attachments']['wrapper']['files'] as $fid => $file_description) {
        if (is_array($file_description)) {
          $form['attachments']['wrapper']['files'][$fid]['description']['#title'] = t('Description');
          $form['attachments']['wrapper']['files'][$fid]['remove']['#title'] = t('Remove');
          $form['attachments']['wrapper']['files'][$fid]['remove']['#description'] = t('If checked, the file will be removed upon save <strong>permanently</strong>.');
          $form['attachments']['wrapper']['files'][$fid]['weight']['#title'] = t('Weight');
          $form['attachments']['wrapper']['files'][$fid]['list']['#title'] = t('List');
          $form['attachments']['wrapper']['files'][$fid]['list']['#description'] = t('If checked, the file will be listed on node view.');
        }
      }
    }
    /////////////////////////////////////////////////////
    // Remove the upload node submit button/handler
    if (module_exists('upload') && isset($form['attachments']['wrapper']['new']['attach'])) {
      unset($form['attachments']['wrapper']['new']['attach']);
    }
  }
}

/**
 * Set redirection of the node form
 *
 * @see node_form_submit()
 * @see drupad_form_alter()
 */
function drupad_node_form_submit($form, &$form_state) {
  // Redirect to the node edit form if it has succeed at saving
  if (isset($form_state['nid'])) {
    $form_state['redirect'] = 'drupad/content-forms/node/'. $form_state['nid'];
  }
}

function drupad_content_get_form_for_node_type($content_type) {
  module_load_include('inc', 'node', 'node.pages');
  $data = array();

  // We need to build the form before getting Errors information
  $data['content'] = drupad_node_add($content_type);
  $data['meta'] = array();
  $data['meta']['errors'] = form_get_errors() ? form_get_errors() : array();

  return $data;
}

function drupad_content_get_form_for_node($nid) {
  module_load_include('inc', 'node', 'node.pages');
  $data = array('meta' => array());

  $node = node_load($nid);

  if (node_access('update', $node, NULL)) {
    // We need to build the form before getting Errors information
    $data['content'] = drupad_node_add($node->type, $node);
    $data['meta']['errors'] = form_get_errors() ? form_get_errors() : array();
  }
  else {
    $data['content'] = array();
    $data['meta']['errors'] = array('node_access' => FALSE);
    watchdog('access denied', '@q accessed via Drupad iOS app', array('@q' => $_GET['q']), WATCHDOG_WARNING);
  }

  return $data;
}

/**
 * Returns a form structure either for a node type or a node
 *
 * @param $type
 *  the content type to retrieve
 * @param $node
 *  the node that should fill the form (can be NULL)
 *
 * @return either a form structured array or FALSE
 */
function drupad_node_add($type, $node = NULL) {
  global $user;

  $types      = node_get_types();
  $type       = isset($type) ? str_replace('-', '_', $type) : NULL;
  $form_id    = $type . '_node_form';
  $form_state = array();

  // If a node type has been specified, validate its existence.
  if (isset($types[$type]) && node_access('create', $type)) {
    // Initialize settings:
    if (!$node) {
      $node = array(
        'uid' => $user->uid,
        'name' => (isset($user->name) ? $user->name : ''),
        'type' => $type,
        'language' => '',
      );
    }

    $form = drupad_get_form($type . '_node_form', $node);
    drupad_fix_node_form_admin_order($form);
    drupad_convert_form_options_to_strict_array($form);

    return $form;
  }

  return FALSE;
}

/**
 * Apply the same modification that theme_node_form does on component order
 *
 * This allows us to present form components in the same order as the HTML UI
 *
 * @see theme_node_form()
 */
function drupad_fix_node_form_admin_order(&$form) {
  if ($form['author']) {
    $form['author']['#weight'] = 99997;
  }
  if (isset($form['options'])) {
    $form['options']['#weight'] = 99998;
  }
  if (isset($form['buttons'])) {
    $form['buttons']['#weight'] = 99999;
  }
}
